unit WinTray;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, ShellApi, Menus, Controls;

{$R *.res}

type
  TWinTray = class(TComponent)
  private
    FWindowHandle: HWND;
    TrayInfo: TNotifyIconDataA;
    FIcon: TIcon;
    FHint: string;
    FVisible: Boolean;
    FPopupMenu: TPopupMenu;
    FOnRightDblClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnRightClick: TNotifyEvent;
    FOnClick: TNotifyEvent;
    FOnMouseMove: TNotifyEvent;
    procedure WndProc(var Msg: TMessage);
    procedure RefreshIconInfo;
    procedure SetIcon(const Value: TIcon);
    procedure SetHint(const Value: string);
    procedure SetVisible(const Value: Boolean);
    procedure AddIcon;
    procedure DelIcon;
    procedure IconNotify(var Msg: TMessage);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
  published
    property Icon: TIcon read FIcon write SetIcon;
    property Hint: string read FHint write SetHint;
    property Visible: Boolean read FVisible write SetVisible;
    property PopupMenu: TPopupMenu read FPopupMenu write FPopupMenu;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnRightClick: TNotifyEvent read FOnRightClick write FOnRightClick;
    property OnRightDblClick: TNotifyEvent read FOnRightDblClick write FOnRightDblClick;
    property OnMouseMove: TNotifyEvent read FOnMouseMove write FOnMouseMove;
  end;

const
  TrayNotify = WM_USER+1;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('fse', [TWinTray]);
end;

{ TWinTray }

procedure TWinTray.AddIcon;
begin
  RefreshIconInfo;
  Shell_NotifyIcon(NIM_ADD,@TrayInfo);
end;

constructor TWinTray.Create(AOwner: TComponent);
begin
  inherited;
  FWindowHandle:=AllocateHWnd(WndProc);
  FIcon:=TIcon.Create;
  FHint:='';
  FVisible:=false;
end;

procedure TWinTray.DelIcon;
begin
  with TrayInfo do
  begin
    Wnd:=FWindowHandle;
    uID:=0;
    uCallbackMessage:=TrayNotify;
    cbSize:=SizeOf(TNotifyIconDataA);
  end;
  Shell_NotifyIcon(NIM_DELETE,@TrayInfo);
end;

destructor TWinTray.Destroy;
begin
  DelIcon;
  Icon.Free;
  DeAllocateHWnd(FWindowHandle);
  inherited;
end;

procedure TWinTray.IconNotify(var Msg: TMessage);
begin
  with Msg do
  begin
    case LParam of
      WM_RBUTTONUP:
      begin
        if Assigned(FPopupMenu)then
        begin
          SetForegroundWindow(FWindowHandle);
          FPopupMenu.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
        end;
        if Assigned(FOnRightClick)then
          FOnRightDblClick(Self);
      end;                                
      WM_RBUTTONDBLCLK:
        if Assigned(FOnRightDblClick)then
          FOnRightDblClick(Self);
      WM_LBUTTONUP:
        if Assigned(FOnClick)then
          FOnClick(Self);
      WM_LBUTTONDBLCLK:
        if Assigned(FOnDblClick)then
          FOnDblClick(Self);
      WM_MOUSEMOVE:
        if Assigned(FOnMouseMove)then
          FOnMouseMove(Self);
    end;
  end;
end;

procedure TWinTray.Refresh;
begin
  RefreshIconInfo;
  Shell_NotifyIcon(NIM_MODIFY,@TrayInfo);
end;

procedure TWinTray.RefreshIconInfo;
begin
  with TrayInfo do
  begin
    Wnd:=FWindowHandle;
    uID:=0;
    uFlags:=NIF_ICON or NIF_MESSAGE or NIF_TIP;
    uCallbackMessage:=TrayNotify;
    hIcon:=Icon.Handle;
    StrPCopy(szTip,FHint);
    cbSize:=SizeOf(TNotifyIconDataA);
  end;
end;

procedure TWinTray.SetHint(const Value: string);
begin
  if Value=FHint then Exit;
  FHint := Value;
  StrPCopy(TrayInfo.szTip,FHint);
  Refresh;
end;

procedure TWinTray.SetIcon(const Value: TIcon);
begin
  FIcon.Assign(Value);
  TrayInfo.hIcon:=Icon.Handle;
  Refresh;
end;

procedure TWinTray.SetVisible(const Value: Boolean);
begin
  if FVisible=Value then Exit;
  FVisible := Value;
  if FVisible then
    AddIcon else
      DelIcon;
end;

procedure TWinTray.WndProc(var Msg: TMessage);
begin
  if Msg.Msg=TrayNotify then
    IconNotify(Msg) else
      Msg.Result:=DefWindowProc(FWindowHandle, Msg.Msg,
      Msg.wParam, Msg.lParam);
end;

end.
